<?php
/* *************************************************
 * File:           OmnisWebClass.php
 * First Created:  04.11.2005
 * Last Modified:  <!--DATE-->14.02.2012<!--/DATE-->
 * Author and (C): Michael Hufschmidt
 * License:        http://creativecommons.org/licenses/by-sa/3.0/de/deed.en_GB
 * *************************************************
 * Content / Description / Requirements / Usage:
 *
 * This class creates HTML sniplets for web-pages to be used together with
 * an Omnis Studio web application server. Omnis-specific parameters are read
 * from an .ini - file. Syntax of the .ini-file: Empty lines and lines
 * starting with a ";" are ignored. Each line contains a parameter name,
 * an "=" sign, and the corresponding parameter value. Parameter names
 * are case sesitive. Parameter values must not contain an "=" sign.
 * *************************************************
 * Optionally further parameters can be passed to the $construct-method of the
 * remote task instance of Omnis Studio. These can be set after instanciating
 * this PHP class by calling the method set_additional_params(...).
 * See example for usage below.
 * *************************************************
 * If this page is called with some of the cgi parameters "port", "lib",
 * "class", "width", "height", these will overwrite the parameters OmnisPort,
 * OmnisLibrary, OmnisClass, OmnisWidth, OmnisHeight from the .ini file
 * or from the constructor. The order or precendece is 1. cgi-params,
 * 2. constructor, 3. ini-file. A link might look like the following:
 *    http://192.168.0.2/wmis/wmis.php?lib=WMIS_2&class=rfMain_2
 * This will replace the settings WMIS and rfMain from the .ini - file
 * by WMIS_2 and rfMain_2. Hence one file wmis.php can by used for different
 * libraries and classes.
 * *************************************************
 * If the AppServer is running under Studio 5 (Unicode) you either need to
 * add a line "OmnisUnicodeServer = 1" in the .ini file or you need to
 * add a cgi-parameter "uniode=1".
 * *************************************************
 * The Omnis App-Server can create a "semaphore"-file for indictating that
 * it is running. To do so, Omnis must - immediately after starting, in the
 * $construct-method of the Startup_Task - create a file (filename for
 * instance "alive.txt") in the directory of the calling PHP script and remove
 * it after its shutdown (in the $destruct-method of the Startup_Task).
 * The plugin-part of the dynamic HTML-Page will then only be created
 * if the file "alive.txt" is present. Otherwise a HTML with a warning is
 * created (default: "<h3>Omnis App. Server is off-line</h3>"). This behaviour
 * is cotrolled by calling the method set_semaphore(...) after instanciating
 * the PHP class (see example for usage below).
 * Warning: The function file_exists() will return FALSE for files
 * inaccessible due to safe mode restrictions. However these files still can
 * be included if they are located in safe_mode_include_dir. Consequently, the
 * recognition of the semaphore file may not always work depending on settings
 * in the /etc/php.ini configuration file.
 * *************************************************
 *
 * Example of an  .ini - file:
 * ---------------------------
; File "v1_wmis.ini" for WMIS, Version 1 / Bernhard Nocht Institut
; Version for Saturn / Terra
; First created 2008-04-11, last modified 2010-07-09

; This ini- file will be interpreted by PHP (by the class OmnisWebClass.php)
; and will create HTML sniplets for web-pages which communicate
; to an Omnis Studio webclient server.
; Author: Michael Hufschmidt

; this is the default one
WebServerURL = http://tigerlogicde.dyndns.biz
; the next two lines are optional. If present they enable automatic
; switching between internal (IP-adress = 192.168.x.x or 127.0.0.1)
; and external usage (all other IP-adresses)
WebServerURL_int = http://192.168.0.2
WebServerURL_ext = http://tigerlogicde.dyndns.biz
WebServerScript = /cgi-bin/nph-omniscgi

; the next two values can be empty. If they are empty or omitted,
; the OmnisServer IP  will be the IP-adress of the calling client
; and  OmnisPort will default to 5912.
OmnisServer = 192.168.0.3
OmnisPort = 5912

; the next four values can be overwritten by parameters passed to
; the constructor OmnisWebClass(...) or by cgi parameters
OmnisLibrary = WMIS
OmnisClass = rfMain
OmnisWidth = 800
OmnisHeight = 600

; if present, sets the name of the semaphore file in OmnisWebClass.php
SemaphoreFile = alive.txt

; enable the next line for a Studio 5 app server
; OmnisUnicodeServer = 1

; The next line controls whether an Auto-Installation (a line with
; Codebase = "..." ) is added to the Active-X for the MS Internet Explorer.
InstallerAutoDownload = 1

; Optionally the Codebase can be reconfigured, the default value is
Codebase = ftp://ftp.omnis.net/Studio501/Win32/WebSmart/ie/omwebie.cab

 * ---------------------------
 * End Example .ini - file
 *
 * *************************************************
 * Example for usage:
 *
<?php
  $phplib = dirname(__FILE__) . '/../phpsub'; // wherever you put lib-classes
  set_include_path(get_include_path() . PATH_SEPARATOR . $phplib);
  require_once('OmnisWebClass.php');
  $x = new OmnisWebClass('v1_wmis.ini', 'rfQuestionaire', 706, 568);

  // Uncomment the next line if you want to use semaphores, the name of the
  // semaphore file can alternatively be set in the ini file.
  // $x->set_semaphore('alive.txt');

  // Uncomment the next lines if you want to provide additional parameters to
  // the $construct-method of the remote task instance in Omnis Studio.
  // Note: In safe mode (variable 'safe_mode = On' in the file /etc/php.ini),
  // the variables PHP_AUTH_USER, PHP_AUTH_PW, and AUTH_TYPE are not available
  // in $_SERVER. Regardless, you can still use REMOTE_USER for the USER.
  // $params = array ();
  // $params['Browser'] = $_SERVER['HTTP_USER_AGENT'];
  // if (isset($_SERVER['PHP_AUTH_USER']))
  //   $params['User'] = $_SERVER['PHP_AUTH_USER'];
  // if (isset($_SERVER['PHP_AUTH_PW']))
  //   $params['Password'] = $_SERVER['PHP_AUTH_PW'];
  // $x->set_additional_params($params);

  $x->display_plugin();
?>


 * *************************************************
 * This class can also be used to create a link with CGI parameters
 * to be evaluated by Omnis. The PHP code would be:
 *
<?php
  $phplib = dirname(__FILE__) . '/../phpsub'; // wherever you put lib-classes
  set_include_path(get_include_path() . PATH_SEPARATOR . $phplib);
  require_once('OmnisWebClass.php');
  $x = new OmnisWebClass('v1_wmis.ini','rtWmis');
  $x->display_link('Abfrage und Ergebnisse','what=sel');
?>
  *
  * This will - with a certain .ini file - create the following HTML-Code:
  *
<a href="http://195.64.103.38/cgi-bin/nph-omniscgi ?OmnisServer=192.168.0.3:5917&OmnisClass=rtWmis &OmnisLibrary=WMIS&what=sel">Abfrage und Ergebnisse</a>


 * *************************************************
 * This class can also be used to create the header of a HTML form with
 * hidden params to be evaluated by Omnis. The PHP code would be:
 *
  ...
  $x->display_html_form_header($o_class, $form_params);
  ...
  * The parameter $o_class is the name of the Omnis remote task which
  * evaluates the HTML form. If omitted it will be taken from the
  * .ini file.
  * The optional parameter $form_params can contain further parameters
  * to be added to the <form ...> tag, for instance
  *      "accept-charset=\"ISO-8859-1\"" or "accept-charset=\"UTF-8\""
  * Example: The php code
  ...
  $x = new OmnisWebClass('stiewi_html_form.ini');
  $ini_array = $x->get_ini_array();
  if (array_key_exists('OmnisUnicodeServer', $ini_array)) {
    $form_params = "accept-charset=\"UTF-8\"";
  } else {
    $form_params = "accept-charset=\"ISO-8859-1\"";
  }
  $x->display_html_form_header("", $form_params);
  *
  * will - with a certain .ini file - create the following HTML-Code:
  *
<form action="/omnis_apache" method="post" enctype="multipart/form-data" accept-charset="ISO-8859-1">
  <input type="hidden" name="OmnisServer" value="192.168.0.8:5912">
  <input type="hidden" name="OmnisLibrary" value="HTML_FORM">
  <input type="hidden" name="OmnisClass" value="rtDynHTML">

 * See also the comments of the function display_html_form_header(...)
 * further down this file.


 * *************************************************
 * This class can also be used for the Studio 5.2+ JavaScript client.
 * *************************************************
 *
 * To create the header part of the JS client the PHP code would be
  $x->display_css_header_omnis_js('/css');
  $x->display_script_header_omnis_js('/scripts');
 * which will create something like
<!-- This makes mobile device support work -->
<meta name="viewport" content="width=device-width">
<!-- The style sheets must be present in the directory /css/ -->
<link type="text/css" href="/css/omn_dlg.css" rel="stylesheet" media="print,screen" />
<link type="text/css" href="/css/omn_menu.css" rel="stylesheet" media="print, screen" />
<link type="text/css" href="/css/smoothness/jquery-ui-1.8.15.custom.css" rel="stylesheet" />
<link type="text/css" href="/css/slick.grid.css" rel="stylesheet" />
<link type="text/css" href="/css/slick.columnpicker.css" rel="stylesheet" />
<link type="text/css" href="/css/slick.pager.css" rel="stylesheet" />
<!-- Must occur after other stylesheets e.g. jquery-ui-1.8.15.custom.css, as it overrides values -->
<link type="text/css" href="/css/omnis.css" rel="stylesheet"/>
<!-- Omnis Studio JavaScript client scripts, must be present in the directory /scripts/ -->
<script type="text/javascript" src="/scripts/omjsclnt.js"></script>
<script type="text/javascript" src="/scripts/omjqclnt.js"></script>
 * The parameters for display_css_header_omnis_js() and
 * display_script_header_omnis_js() are optional, if omitted the directories
 * css and scrips must be relative to the current page. See further comments
 * in the functions further down this page.
 *
 *
 * To create the body part of a JS client the PHP code would be:
  $x->display_client_omnis_js();
 * which will create something like
  <div id="omnisobject1" WebServerUrl="http://192.168.0.4/omnis_apache" OmnisServerAndPort="192.168.0.8:5914" OmnisLibrary="ScriptClient52_Test" OmnisClass="rfTest"></div>
 * optional parameters determine the number of the omnisobject, the OmnisClass
 * and a style.See further commentsin the function further down this page.
 *
 *
 * The PHP-Code for complete PHP + HTML-Page could be
<html>
<head>
<title>Studio 5.2 Test</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<?php
  $phplib = dirname(__FILE__) . '/../phpsub'; // wherever you put lib-classes
  set_include_path(get_include_path() . PATH_SEPARATOR . $phplib);
  require_once('OmnisWebClass.php');
  $x = new OmnisWebClass('ScriptClient52_Test.ini', 'rfTest');
  $params = array ("A1", "B1", "C1", "D1");
  $x->set_additional_params($params);
  $x->display_header_body_omnis_js('/css','/scripts','margin:0;');
?>
<h1>Test JavaScript Client Studio 5.2</h1>
<?php
  $x->display_client_omnis_js();
?>
</body>
</html>
 *
 *
 * Which will create something like
<html>
<head>
<title>Studio 5.2 Test</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">

<!-- This makes mobile device support work -->
<meta name="viewport" content="width=device-width">

<!-- The style sheets must be present in the directory /css/ -->
<link type="text/css" href="/css/omn_dlg.css" rel="stylesheet" media="print,screen" />
<link type="text/css" href="/css/omn_menu.css" rel="stylesheet" media="print, screen" />
<link type="text/css" href="/css/smoothness/jquery-ui-1.8.15.custom.css" rel="stylesheet" />

<link type="text/css" href="/css/slick.grid.css" rel="stylesheet" />
<link type="text/css" href="/css/slick.columnpicker.css" rel="stylesheet" />
<link type="text/css" href="/css/slick.pager.css" rel="stylesheet" />
<!-- Must occur after other stylesheets e.g. jquery-ui-1.8.15.custom.css, as it overrides values -->
<link type="text/css" href="/css/omnis.css" rel="stylesheet"/>

<!-- Omnis Studio JavaScript client scripts, must be present in the directory /scripts/ -->
<script type="text/javascript" src="/scripts/omjsclnt.js"></script>
<script type="text/javascript" src="/scripts/omjqclnt.js"></script>

</head>
<body onload="jOmnis.onLoad()" onunload="jOmnis.onUnload()" style="margin:0;">
<h1>Test JavaScript Client Studio 5.2</h1>
<div id="omnisobject1" WebServerUrl="http://192.168.0.4/omnis_apache" OmnisServerAndPort="192.168.0.8:5914" OmnisLibrary="ScriptClient52_Test" OmnisClass="rfTest" param1="A1" param2="B1" param3="C1" param4="D1"></div>
</body>
</html>

 * *************************************************/

class OmnisWebClass {
  var $ini_array;             // contains the content of the ini-file
  var $WebServerURL;          // this will store the effective WebServerURL
                              // either WebServerURL, or WebServerURL_ext,
                              // or WebServerURL_int.
  var $codebase;              // contains the codebase for Auto-Installer
  var $additional_params;     // contains an associative array of max. 10
                              // elements which will be additionally passed to
                              // the $construct-method of the remote task
                              // instance of Omnis Studio
  var $semaphore_file;        // name of a file which should only exist if the
                              // Omnis App.-Server is running
  var $no_semaphore;          // contains HTML to be output instead of the
                              // plugin-area if the file $semaphore_file
                              // does not exist.

// *************************************************
// Constructor(s), Setters and Getters
// *************************************************

  function OmnisWebClass($inifilename, $omnis_class='', $width=0, $height=0) {
  // This is the constructor.
  // The parameters $omnis_class, $width, $height are optional.
  // If they are omitted, they are taken from the ini file, otherwise
  // they will override the corresponding values from the .ini file.
    if (file_exists($inifilename)) {
      $this->ini_array = parse_ini_file($inifilename);
      $ip = $_SERVER["REMOTE_ADDR"];
      // echo "<p>IP-Adresse = $ip</p>"; for testing
      if (empty($this->ini_array['OmnisServer']))
          $this->ini_array['OmnisServer'] = $ip;
      if (empty($this->ini_array['OmnisPort']))
          $this->ini_array['OmnisPort'] = 5912;
      if (isset($_REQUEST['port']))
          $this->ini_array['OmnisPort'] = $_REQUEST['port'];
      if (isset($_REQUEST['lib']))
        $this->ini_array['OmnisLibrary'] = $_REQUEST['lib'];
      if (!empty($omnis_class))
          $this->ini_array['OmnisClass'] = $omnis_class;
      if (isset($_REQUEST['class']))
        $this->ini_array['OmnisClass'] = $_REQUEST['class'];
      // if (!array_key_exists('OmnisUnicodeServer', $this->ini_array))
        // $this->ini_array['OmnisUnicodeServer'] = 0;
      if (!empty($width))
           $this->ini_array['OmnisWidth'] = $width;
      if (isset($_REQUEST['width']))
           $this->ini_array['OmnisWidth'] = $_REQUEST['width'];
      if (!empty($height))
           $this->ini_array['OmnisHeight'] = $height;
      if (isset($_REQUEST['height']))
           $this->ini_array['OmnisHeight'] = $_REQUEST['height'];
      if (isset($_REQUEST['unicode']))
        $this->ini_array['OmnisUnicodeServer'] = $_REQUEST['unicode'];
      $this->WebServerURL = $this->ini_array['WebServerURL'];
      if (($ip == '127.0.0.1') || (substr($ip,0,8) == '192.168.')) {
          if (!empty($this->ini_array['WebServerURL_int']))
            $this->WebServerURL = $this->ini_array['WebServerURL_int'];
      } else {
          if (!empty($this->ini_array['WebServerURL_ext']))
            $this->WebServerURL = $this->ini_array['WebServerURL_ext'];
      }
      if (!empty($this->ini_array['Codebase'])) {
        $this->codebase = $this->ini_array['Codebase'];
      } else {
        $this->codebase = "ftp://ftp.omnis.net/Studio501/Win32/WebSmart/ie/omwebie.cab";
      }
      $this->set_semaphore();
     } else {
      echo "<p>Error: Could not open the ini-file \"$inifilename\"</p>\r\n";
    }
  }  // end constructor OmnisWebClass(...)

  function set_codebase($codebase) {
    $this->codebase = $codebase;
  }  // end function set_codebase(...)

  function set_additional_params($additional_params) {
    if (is_array($additional_params))
      $this->additional_params = $additional_params;
  }  // end function set_additional_params(...)

  function set_semaphore($semaphore_file = '', $no_semaphore = '') {
    if (!empty($semaphore_file)) {
      $this->semaphore_file = $semaphore_file;
    } else {
      if (!empty($this->ini_array['SemaphoreFile'])) {
        $this->semaphore_file = $this->ini_array['SemaphoreFile'];
      }
    }
    if (!empty($no_semaphore)) {
      $this->no_semaphore = $no_semaphore;
    } else {
      $this->no_semaphore = "<h3>Omnis App. Server with Library \"" .
        $this->ini_array['OmnisLibrary'] . "\" is off-line!</h3>";
    }
  }  // end function set_semaphore(...)

  function get_ini_array() {
    return $this->ini_array;
  }  // end function get_ini_array()

// *************************************************
// Output Methods
// *************************************************

  function display_info($leading_tag = '', $trailing_tag = '') {
  // Displays general info about the server and the .ini - file,
  // the optional parameters could be something like '<h3>', '</h3>'.
    if (!empty($leading_tag)) {
	  echo $leading_tag;
	}
	echo "Web-Server = " . $_SERVER['HTTP_HOST']  . ":" . $_SERVER['SERVER_PORT'];
    echo ", Remote Client = " . $_SERVER['REMOTE_ADDR'] . "<br>\r\n";
    echo "  Omnis App. Server  = " . $this->ini_array['OmnisServer'] . ":" . $this->ini_array['OmnisPort'] . "<br>\r\n";
	echo "  OmnisLibrary = " . $this->ini_array['OmnisLibrary'] . ", OmnisClass = " . $this->ini_array['OmnisClass'] . "<br>\r\n";
    if (!empty($trailing_tag)) {
	  echo $trailing_tag . "\r\n";
	}
  }

  function make_http_get_parms() {
  // returns a string like
  //    http://212.20.132.162:8080/cgi-bin/nph-omniscgi?
  //        OmnisServer=192.168.0.3:5924&
  //        OmnisLibrary=WMIS&OmnisClass=rtInterface
  // where all the parameters are taken from instance variables of this class
  //
    extract($this->ini_array);
    $url = "$this->WebServerURL" .
      "$WebServerScript?OmnisServer=$OmnisServer:$OmnisPort" .
      "&OmnisLibrary=$OmnisLibrary&OmnisClass=$OmnisClass";
    if (array_key_exists('OmnisUnicodeServer', $this->ini_array))
      $url .= "&OmnisUnicodeServer=$OmnisUnicodeServer";
    return $url;
  }  // End of  make_http_get_parms()

  function display_link($linktext, $further_params = '') {
  // creates the link to an Omnis Application like
  // <a href="http://195.64.103.38/cgi-bin/nph-omniscgi
  // ?OmnisServer=192.168.0.3:5917&OmnisClass=rtWmis
  // &OmnisLibrary=WMIS&what=sel">Abfrage und Ergebnisse</a>
  // The parameter $further_params is optional.
    $url = $this->make_http_get_parms();
    if (empty($further_params)) {
      echo "<a href=\"$url\">$linktext</a>";
    } else {
      echo "<a href=\"$url&$further_params\">$linktext</a>";
    }
  }  // End of display_link()

  function display_plugin() {
  // Creates the Omnis plugin-part of a HTML page like:
  // <!-- The following is created dynamically by PHP -->
  // <!-- Use for Microsoft Internet Explorer -->
  // <!-- ActiveX object START-->
  //
  // <object classid="clsid:13510606-30FA-11D2-B383-444553540000"
  //   width="628" height="568">
  //   <param name="_Version" value="65536">
  //   <param name="_ExtentX" value="7161">
  //   <param name="_ExtentY" value="7373">
  //   <param name="_StockProps" value="0">
  //   <param name="OmnisServer" value="192.168.0.3:5912">
  //   <param name="OmnisLibrary" value="WMIS">
  //   <param name="OmnisClass" value="rfQuestionaire">
  //   <param name="OmnisUnicodeServer" value="1">
  //   <param name="WebServerURL" value="http://192.168.0.2">
  //   <param name="WebServerScript" value="/cgi-bin/nph-omniscgi">
  //
  // <!-- Use for Netscape, Firefox and others (Windows, Linux and Mac) -->
  // <!-- Netscape object START-->
  // <embed type="application/OMNIS-RCC-plugin" name="rcc1"
  //   width="628" height="568" OmnisServer="192.168.0.3:5912"
  //   OmnisLibrary="WMIS" OmnisClass="rfQuestionaire" OmnisUnicodeServer="1"
  //   WebServerUrl="http://192.168.0.2"
  //   WebServerScript="/cgi-bin/nph-omniscgi">
  // <!-- Netscape object END-->
  //
  // </object>
  // <!-- ActiveX object END-->
  // <!-- End of PHP script -->

    // Check Semaphores: Is a semaphore file defined and is it present?
    if (!empty($this->semaphore_file))
      if(!file_exists($this->semaphore_file)) {
        echo $this->no_semaphore . "\r\n" ;
      return;
    }
   // Omnis server seems to be running at this point
    extract($this->ini_array);
    echo "<!-- The following is created dynamically by PHP -->\r\n";
    echo "<!-- Use for Microsoft Internet Explorer -->\r\n";
    echo "<!-- ActiveX object START -->\r\n";
    echo "<object classid=\"clsid:13510606-30FA-11D2-B383-444553540000\"\r\n";
    if (isset($InstallerAutoDownload)) {
      if ($InstallerAutoDownload) {
        echo "  codebase=\"$this->codebase\"\r\n";
      }
    }
    echo "  width=\"$OmnisWidth\" height=\"$OmnisHeight\">\r\n";
    echo "  <param name=\"_Version\" value=\"65536\">\r\n";
    echo "  <param name=\"_ExtentX\" value=\"7161\">\r\n";
    echo "  <param name=\"_ExtentY\" value=\"7373\">\r\n";
    echo "  <param name=\"_StockProps\" value=\"0\">\r\n";
    echo "  <param name=\"OmnisServer\"" .
              " value=\"$OmnisServer:$OmnisPort\">\r\n";
    echo "  <param name=\"OmnisLibrary\"" .
              " value=\"$OmnisLibrary\">\r\n";
    echo "  <param name=\"OmnisClass\" value=\"$OmnisClass\">\r\n";
    if (array_key_exists('OmnisUnicodeServer', $this->ini_array))
      echo "  <param name=\"OmnisUnicodeServer\" value=\"$OmnisUnicodeServer\">\r\n";
    echo "  <param name=\"WebServerURL\" value=\"$this->WebServerURL\">\r\n";
    echo "  <param name=\"WebServerScript\"" .
              " value=\"$WebServerScript\">\r\n";
    if (is_array($this->additional_params)) {
    // create additional Parameters, syntax for Internet Explorer
      $keys = array_keys($this->additional_params);
      for ($p = 1; $p <= count($this->additional_params); $p++) {
        $key = $keys[$p-1];
        $value = $this->additional_params[$key];
        echo "  <param name=\"Param$p\" value=\"$key=$value\">\r\n";
      }
    }
    echo "<!-- Use for Netscape, Firefox and others" .
              " (Windows, Linux and Mac) -->\r\n";
    echo "<!-- Netscape object START-->\r\n";
    echo "<embed type=\"application/OMNIS-RCC-plugin\" name=\"rcc1\"" .
              " width=\"$OmnisWidth\" height=\"$OmnisHeight\"" .
              " OmnisServer=\"$OmnisServer:$OmnisPort\"" .
              " OmnisLibrary=\"$OmnisLibrary\" OmnisClass=\"$OmnisClass\"";
    if (array_key_exists('OmnisUnicodeServer', $this->ini_array))
      echo " OmnisUnicodeServer=\"$OmnisUnicodeServer\"";
    echo " WebServerUrl=\"$this->WebServerURL\"" .
              " WebServerScript=\"$WebServerScript\"";
    if (is_array($this->additional_params)) {
    // create additional Parameters, syntax for Firefox & Co.
      $keys = array_keys($this->additional_params);
      for ($p = 1; $p <= count($this->additional_params); $p++) {
        $key = $keys[$p-1];
        $value = $this->additional_params[$key];
        echo " Param$p=\"$key=$value\"";
      }  // end for
    }  // end if
    echo ">\r\n";
    echo "<!-- Netscape object END-->\r\n";
    echo "</object>\r\n";
    echo "<!-- ActiveX object END-->\r\n";
    echo "<!-- End of PHP script -->\r\n";
  }  // End of display_plugin()

  function display_html_form_header($o_class = '', $form_params = '') {
    // this will display the header part of a HTML form with hidden params,
    // so that the content of the HTML form will be evaluated by a remote
    // task within Omnis.
    // The parameter $o_class is the name of the Omnis remote task which
    // evaluates the HTML form. If omitted it will be taken from the
    // .ini file.
    // The optional parameter $form_params can contain further parameters
    // to be added to the <form ...> tag.
    extract($this->ini_array);
    if (!empty($o_class)) {
      $OmnisClass = $o_class;
    }
    echo "<form action=\"$WebServerScript\" method=\"post\"" .
         " enctype=\"multipart/form-data\" $form_params>\r\n";
    echo "  <input type=\"hidden\" name=\"OmnisServer\"" .
         " value=\"$OmnisServer:$OmnisPort\">\r\n";
    echo "  <input type=\"hidden\" name=\"OmnisLibrary\"" .
         " value=\"$OmnisLibrary\">\r\n";
    echo "  <input type=\"hidden\" name=\"OmnisClass\"" .
         " value=\"$OmnisClass\">\r\n";
  }  // End of display_html_form_header

// *************************************************
// Output Methods for the Omnis JavaScrip client
// *************************************************

  function display_css_header_omnis_js($css_dir = ''){
  // will display the header lines concerning sytle sheets, for instance
  //   <!-- This makes mobile device support work -->
  //   <meta name="viewport" content="width=device-width">
  //   <!-- The style sheets must be present in the directory css/ -->
  //   <link type="text/css" href="css/omn_dlg.css" rel="stylesheet" media="print,screen" />
  //   <link type="text/css" href="css/omn_menu.css" rel="stylesheet" media="print, screen" />
  //   <link type="text/css" href="css/smoothness/jquery-ui-1.8.15.custom.css" rel="stylesheet" />
  //   <link type="text/css" href="css/slick.grid.css" rel="stylesheet" />
  //   <link type="text/css" href="css/slick.columnpicker.css" rel="stylesheet" />
  //   <link type="text/css" href="css/slick.pager.css" rel="stylesheet" />
  //   <!-- Must occur after other stylesheets e.g. jquery-ui-1.8.15.custom.css, as it overrides values -->
  //   <link type="text/css" href="css/omnis.css" rel="stylesheet"/>
  // if the style sheets are not in a directory css below
  // this page, the parameter $css_dir must point to this directory,
  // for instance $css_dir='/css' when using an absolute path.
    if (empty($css_dir)) {
      $css_dir = "css";
    }
    echo "\r\n<!-- This makes mobile device support work -->\r\n";
    echo "<meta name=\"viewport\" content=\"width=device-width\">\r\n";
    echo "\r\n<!-- The style sheets must be present in the directory $css_dir/ -->\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/omn_dlg.css\" rel=\"stylesheet\" media=\"print,screen\" />\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/omn_menu.css\" rel=\"stylesheet\" media=\"print, screen\" />\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/smoothness/jquery-ui-1.8.15.custom.css\" rel=\"stylesheet\" />\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/slick.grid.css\" rel=\"stylesheet\" />\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/slick.columnpicker.css\" rel=\"stylesheet\" />\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/slick.pager.css\" rel=\"stylesheet\" />\r\n";
    echo "<!-- Must occur after other stylesheets e.g. jquery-ui-1.8.15.custom.css, as it overrides values -->\r\n";
    echo "<link type=\"text/css\" href=\"$css_dir/omnis.css\" rel=\"stylesheet\"/>\r\n\r\n";
  }  // end display_css_header_omnis_jss

  function display_script_header_omnis_js($script_dir = ''){
  // will display the header lines concerning Java Scripts, for instance
  //    <!-- Omnis Studio JavaScript client scripts, must be present in the directory scripts/ -->
  //    <script type="text/javascript" src="scripts/omjsclnt.js"></script>
  //    <script type="text/javascript" src="scripts/omjqclnt.js"></script>
  // if the source of the java scripts is not in a directory scripts below
  // this page, the parameter $script_dir must point to this directory,
  // for instance $script_dir='/scripts' when using an absolute path.
    if (empty($script_dir)) {
      $script_dir = "scripts";
    }
    echo "<!-- Omnis Studio JavaScript client scripts, must be present in the directory $script_dir/ -->\r\n";
    echo "<script type=\"text/javascript\" src=\"$script_dir/omjsclnt.js\"></script>\r\n";
    echo "<script type=\"text/javascript\" src=\"$script_dir/omjqclnt.js\"></script>\r\n\r\n";
  }  // end display_script_header_omnis_js

  function display_body_tag_omnis_js( $myStyle = '') {
    // will display something like
    //   <body onload="jOmnis.onLoad()" onunload="jOmnis.onUnload()" style="margin:0;">
    // the style="..." will only be shown if a parameter $myStyle is provided.
    echo "<body onload=\"jOmnis.onLoad()\" onunload=\"jOmnis.onUnload()\"";
    if (!empty($myStyle)) {
      echo " style=\"$myStyle\"";
    }
    echo ">\r\n";
  }  //end display_body_tag_omnis_js

  function display_header_body_omnis_js($css_dir = '', $script_dir = '', $myStyle = '') {
    // will display the output of display_css_header_omnis_js($css_dir),
    // the output of  display_script_header_omnis_js($script_dir)
    // a    </head>
    // and he output of display_body_tag_omnis_js( $myStyle)
    $this->display_css_header_omnis_js($css_dir);
    $this->display_script_header_omnis_js($script_dir);
    echo "</head>\r\n";
    $this->display_body_tag_omnis_js($myStyle);
  }  // end display_header_body_omnis_js

  function display_client_omnis_js($id = 1, $myClass = '', $myStyle = '') {
    // this will display the body part of the Omnis JavaScript client, e.g.
    // <div id="omnisobject1"
    //    style="position:absolute;top:0px;left:0px"
    //    WebServerURL="http://192.168.0.4/cgi-bin/nph-omniscgi"
    //    OmnisServerAndPort="192.168.0.8:5914"
    //    OmnisLibrary="ScriptClient52_Test" OmnisClass="rfTest"
    //    param1="" param2="" ...>
    //  </div>
    // the additional parameters param1, param2, ... are taken from
    // $this->additional_params.
    // If the parameter $myClass is provided, it will override the OmnisClass
    // from the ini file or from the constructor.
    // The style="..." will only be shown if a parameter $myStyle is provided.

    // Check Semaphores: Is a semaphore file defined and is it present?
    if (!empty($this->semaphore_file))
      if(!file_exists($this->semaphore_file)) {
        echo $this->no_semaphore . "\r\n" ;
      return;
    }
   // Omnis server seems to be running at this point
    extract($this->ini_array);
    if (!empty($myClass)) {
      $OmnisClass = $myClass;
    }  // end if
    echo "<div id=\"omnisobject" . $id . "\"" ;
    if (!empty($myStyle)) {
      echo " style=\"$myStyle\"";
    }
    echo " WebServerUrl=\"" . $this->WebServerURL . $WebServerScript . "\"" .
      " OmnisServerAndPort=\"" . $OmnisServer . ":" . $OmnisPort . "\"" .
      " OmnisLibrary=\"" . $OmnisLibrary . "\"" .
      " OmnisClass=\"" . $OmnisClass . "\"";
    if (is_array($this->additional_params)) {
    // create additional Parameters
      $keys = array_keys($this->additional_params);
      for ($p = 1; $p <= count($this->additional_params); $p++) {
        $key = $keys[$p-1];
        $value = $this->additional_params[$key];
        echo " param" . $p . "=\"" . $value . "\"";
      }  // end for
    }  // end if
      echo "></div>\r\n";
  }  // end display_client_omnis_js

}  // End of class definition OmnisWebClass
?>